@@ -1,39 +1,30 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import with_statement
-
-import re
 import os
-import sys
+import re
 import shutil
+import sys
 import tempfile
-
-from functools import wraps
 from functools import partial
+from functools import wraps
+from unittest import TestCase
 
-try:
-    from unittest2 import TestCase
-except ImportError:
-    from unittest import TestCase
+from chameleon.exc import RenderError
+from chameleon.tales import DEFAULT_MARKER
 
 
-from chameleon.utils import byte_string
-
-
-class Message(object):
+class Message:
     def __str__(self):
         return "message"
 
 
 class ImportTestCase(TestCase):
     def test_pagetemplates(self):
-        from chameleon import PageTemplate
-        from chameleon import PageTemplateFile
-        from chameleon import PageTemplateLoader
+        from chameleon import PageTemplate  # noqa: F401 unused
+        from chameleon import PageTemplateFile  # noqa: F401 unused
+        from chameleon import PageTemplateLoader  # noqa: F401 unused
 
     def test_pagetexttemplates(self):
-        from chameleon import PageTextTemplate
-        from chameleon import PageTextTemplateFile
+        from chameleon import PageTextTemplate  # noqa: F401 unused
+        from chameleon import PageTextTemplateFile  # noqa: F401 unused
 
 
 class TemplateFileTestCase(TestCase):
@@ -94,18 +85,18 @@ class TemplateFileTestCase(TestCase):
         template = self._class("___does_not_exist___")
         try:
             template.cook_check()
-        except IOError:
-            exc = sys.exc_info()[1]
+        except OSError as exc:
             self.assertEqual(
                 os.getcwd(),
                 os.path.dirname(exc.filename)
-                )
+            )
         else:
             self.fail("Expected OSError.")
 
 
 class RenderTestCase(TestCase):
     root = os.path.dirname(__file__)
+    maxDiff = 4096
 
     def find_files(self, ext):
         inputs = os.path.join(self.root, "inputs")
@@ -117,10 +108,10 @@ class RenderTestCase(TestCase):
             path = os.path.join(inputs, filename)
 
             # if there's no output file, treat document as static and
-            # expect intput equal to output
+            # expect input equal to output
             import glob
             globbed = tuple(glob.iglob(os.path.join(
-                outputs, "%s*%s" % (name.split('-', 1)[0], ext))))
+                outputs, "{}*{}".format(name.split('-', 1)[0], ext))))
 
             if not globbed:
                 self.fail("Missing output for: %s." % name)
@@ -163,9 +154,8 @@ class ZopePageTemplatesTest(RenderTestCase):
             def wrapper(self):
                 from chameleon.exc import TemplateError
                 try:
-                    template = self.from_string(body)
-                except TemplateError:
-                    exc = sys.exc_info()[1]
+                    self.from_string(body)
+                except TemplateError as exc:
                     return func(self, body, exc)
                 else:
                     self.fail("Expected exception.")
@@ -181,7 +171,7 @@ class ZopePageTemplatesTest(RenderTestCase):
             self.from_string,
             """<tal:block replace='bad /// ' />""",
             strict=True
-            )
+        )
 
     def test_syntax_error_in_non_strict_mode(self):
         from chameleon.exc import ExpressionError
@@ -191,12 +181,79 @@ class ZopePageTemplatesTest(RenderTestCase):
 
         try:
             template()
-        except ExpressionError:
-            exc = sys.exc_info()[1]
+        except ExpressionError as exc:
             self.assertTrue(body[exc.offset:].startswith('bad ///'))
         else:
             self.fail("Expected exception")
 
+    def test_exists_error_leak(self):
+        body = '''\
+        <?xml version="1.0"?>
+        <root>
+        <one tal:condition="exists: var_does_not_exists" />
+        <two tal:attributes="my_attr dict()['key-does-not-exist']" />
+        </root>'''
+        template = self.from_string(body, strict=False)
+        try:
+            template()
+        except RenderError as exc:
+            self.assertNotIn('var_does_not_exists', str(exc))
+        else:
+            self.fail("Expected exception")
+
+    def test_sys_exc_info_is_clear_after_pipe(self):
+        body = (
+            '<div tal:content="y|nothing"></div><span tal:content="error()" />'
+        )
+        template = self.from_string(body, strict=False)
+
+        got = template.render(error=sys.exc_info)
+        self.assertTrue('<span>(None, None, None)</span>' in got)
+
+    def test_render_macro_include_subtemplate_containing_error(self):
+        macro_inner = self.from_string(
+            '''<two tal:attributes="my_attr dict()['key-does-not-exist']" />'''
+        )
+        macro_wrap = self.from_string(
+            '''<one tal:content="macro_inner()" />''')
+        template = self.from_string(
+            '''
+            <tal:defs define="translate string:">
+              <span i18n:translate="">foo</span>
+              <metal:macro use-macro="macro" />
+            </tal:defs>
+            ''')
+        try:
+            template(macro=macro_wrap, macro_inner=macro_inner)
+        except RenderError as exc:
+            self.assertTrue(isinstance(exc, KeyError))
+            self.assertIn(''''key-does-not-exist'
+
+ - Expression: "dict()['key-does-not-exist']"
+ - Filename:   <string>
+ - Location:   (line 1: col 29)
+ - Expression: "macro_inner()"
+ - Filename:   <string>
+ - Location:   (line 1: col 18)
+ - Expression: "macro"
+ - Filename:   <string>
+ - Location:   (line 4: col 38)
+''', str(exc))
+        else:
+            self.fail("Expected exception")
+
+    def test_render_error_macro_include(self):
+        body = """<metal:block use-macro='"bad"' />"""
+        template = self.from_string(body, strict=False)
+
+        try:
+            template()
+        except RenderError as exc:
+            self.assertTrue(isinstance(exc, AttributeError))
+            self.assertIn('bad', str(exc))
+        else:
+            self.fail("Expected exception")
+
     @error("""<tal:dummy attributes=\"dummy 'dummy'\" />""")
     def test_attributes_on_tal_tag_fails(self, body, exc):
         self.assertTrue(body[exc.offset:].startswith('dummy'))
@@ -209,6 +266,19 @@ class ZopePageTemplatesTest(RenderTestCase):
     def test_repeat_syntax_error_message(self, body, exc):
         self.assertTrue(body[exc.offset:].startswith('key,value'))
 
+    @error('''<tal:dummy><p i18n:translate="mymsgid">
+            <span i18n:name="repeat"/><span i18n:name="repeat"/>
+            </p></tal:dummy>''')
+    def test_repeat_i18n_name_error(self, body, exc):
+        self.assertTrue(body[exc.offset:].startswith(
+            'repeat'), body[exc.offset:])
+
+    @error('''<tal:dummy>
+            <span i18n:name="not_in_translation"/>
+            </tal:dummy>''')
+    def test_i18n_name_not_in_translation_error(self, body, exc):
+        self.assertTrue(body[exc.offset:].startswith('not_in_translation'))
+
     def test_encoded(self):
         filename = '074-encoded-template.pt'
         with open(os.path.join(self.root, 'inputs', filename), 'rb') as f:
@@ -223,10 +293,21 @@ class ZopePageTemplatesTest(RenderTestCase):
 
         self.from_string(body)
 
+    def test_recursion_error(self):
+        template = self.from_string(
+            '<div metal:define-macro="main" '
+            'metal:use-macro="template.macros.main" />'
+        )
+        self.assertRaises(RecursionError, template)
+        try:
+            template()
+        except RecursionError as exc:
+            self.assertFalse(isinstance(exc, RenderError))
+
     def test_unicode_decode_error(self):
         template = self.from_file(
             os.path.join(self.root, 'inputs', 'greeting.pt')
-            )
+        )
 
         string = native = "the artist formerly known as ƤŗíƞĆě"
         try:
@@ -244,8 +325,7 @@ class ZopePageTemplatesTest(RenderTestCase):
 
         try:
             template(name=name)
-        except UnicodeDecodeError:
-            exc = sys.exc_info()[1]
+        except UnicodeDecodeError as exc:
             formatted = str(exc)
 
             # There's a marker under the expression that has the
@@ -276,7 +356,7 @@ class ZopePageTemplatesTest(RenderTestCase):
         self.assertEqual(
             rendered,
             string.replace('${text}', text)
-            )
+        )
 
     def test_custom_encoding_for_str_or_bytes_in_attributes(self):
         string = '<img tal="Тест${text}" />'
@@ -299,17 +379,28 @@ class ZopePageTemplatesTest(RenderTestCase):
         self.assertEqual(
             rendered,
             string.replace('${text}', text)
-            )
+        )
 
     def test_null_translate_function(self):
         template = self.from_string('${test}', translate=None)
         rendered = template(test=object())
         self.assertTrue('object' in rendered)
 
+    def test_on_error_handler(self):
+        exc = []
+        handler = exc.append
+        template = self.from_string(
+            '<tal:block on-error="string:error">${test}</tal:block>',
+            on_error_handler=handler
+        )
+        template()
+        self.assertEqual(len(exc), 1)
+        self.assertEqual(exc[0].__class__.__name__, "NameError")
+
     def test_object_substitution_coerce_to_str(self):
         template = self.from_string('${test}', translate=None)
 
-        class dummy(object):
+        class dummy:
             def __repr__(inst):
                 self.fail("call not expected")
 
@@ -322,13 +413,13 @@ class ZopePageTemplatesTest(RenderTestCase):
     def test_repr(self):
         template = self.from_file(
             os.path.join(self.root, 'inputs', 'hello_world.pt')
-            )
+        )
         self.assertTrue(template.filename in repr(template))
 
     def test_underscore_variable(self):
         template = self.from_string(
             "<div tal:define=\"_dummy 'foo'\">${_dummy}</div>"
-            )
+        )
         self.assertTrue(template(), "<div>foo</div>")
 
     def test_trim_attribute_space(self):
@@ -345,44 +436,107 @@ class ZopePageTemplatesTest(RenderTestCase):
             document, trim_attribute_space=True)()
 
         self.assertEqual(result1.count(" "), 49)
-        self.assertEqual(result2.count(" "), 3)
+        self.assertEqual(result2.count(" "), 4)
+        self.assertTrue(" />" in result1)
+        self.assertTrue(" />" in result2)
 
     def test_exception(self):
         from traceback import format_exception_only
 
         template = self.from_string(
             "<div tal:define=\"dummy foo\">${dummy}</div>"
-            )
+        )
         try:
             template()
-        except:
-            exc = sys.exc_info()[1]
+        except Exception as exc:
+            self.assertIn(RenderError, type(exc).__bases__)
             formatted = str(exc)
             self.assertFalse('NameError:' in formatted)
             self.assertTrue('foo' in formatted)
-            self.assertTrue('(1:23)' in formatted)
+            self.assertTrue('(line 1: col 23)' in formatted)
 
             formatted_exc = "\n".join(format_exception_only(type(exc), exc))
             self.assertTrue('NameError: foo' in formatted_exc)
         else:
             self.fail("expected error")
 
+    def test_create_formatted_exception(self):
+        from chameleon.utils import create_formatted_exception
+
+        exc = create_formatted_exception(NameError('foo'), NameError, str)
+        self.assertEqual(exc.args, ('foo', ))
+
+        class MyNameError(NameError):
+            def __init__(self, boo):
+                NameError.__init__(self, boo)
+                self.bar = boo
+
+        exc = create_formatted_exception(MyNameError('foo'), MyNameError, str)
+        self.assertEqual(exc.args, ('foo', ))
+        self.assertEqual(exc.bar, 'foo')
+
+    def test_create_formatted_exception_no_subclass(self):
+        from chameleon.utils import create_formatted_exception
+
+        class DifficultMetaClass(type):
+            def __init__(self, class_name, bases, namespace):
+                if not bases == (BaseException, ):
+                    raise TypeError(bases)
+
+        Difficult = DifficultMetaClass(
+            'Difficult', (BaseException, ), {'args': ()})
+
+        exc = create_formatted_exception(Difficult(), Difficult, str)
+        self.assertEqual(exc.args, ())
+
+    def test_error_handler_makes_safe_copy(self):
+        calls = []
+
+        class TestException(Exception):
+            def __init__(self, *args, **kwargs):
+                calls.append((args, kwargs))
+
+        def _render(stream, econtext, rcontext):
+            exc = TestException('foo', bar='baz')
+            rcontext['__error__'] = ('expression', 1, 42, 'test.pt', exc),
+            raise exc
+
+        template = self.from_string("")
+        template._render = _render
+        try:
+            template()
+        except TestException as exc:
+            self.assertEqual(calls, [(('foo', ), {'bar': 'baz'})])
+            formatted = str(exc)
+            self.assertTrue('TestException' in formatted)
+            self.assertTrue('"expression"' in formatted)
+            self.assertTrue('(line 1: col 42)' in formatted)
+        else:
+            self.fail("unexpected error")
+
     def test_double_underscore_variable(self):
         from chameleon.exc import TranslationError
         self.assertRaises(
             TranslationError, self.from_string,
             "<div tal:define=\"__dummy 'foo'\">${__dummy}</div>",
-            )
+        )
+
+    def test_disable_comment_interpolation(self):
+        template = self.from_string(
+            '<!-- ${"Hello world"} -->',
+            enable_comment_interpolation=False
+        )
+        self.assertEqual(template(), '<!-- ${"Hello world"} -->')
 
     def test_compiler_internals_are_disallowed(self):
         from chameleon.compiler import COMPILER_INTERNALS_OR_DISALLOWED
         from chameleon.exc import TranslationError
 
         for name in COMPILER_INTERNALS_OR_DISALLOWED:
-            body = "<d tal:define=\"%s 'foo'\">${%s}</d>" % (name, name)
+            body = "<d tal:define=\"{} 'foo'\">${{{}}}</d>".format(name, name)
             self.assertRaises(TranslationError, self.from_string, body)
 
-    def test_fast_translate_mapping(self):
+    def test_simple_translate_mapping(self):
         template = self.from_string(
             '<div i18n:translate="">'
             '<span i18n:name="name">foo</span>'
@@ -404,51 +558,62 @@ class ZopePageTemplatesTest(RenderTestCase):
         self.assertTrue('foo' in result)
         self.assertTrue('foo' in result)
 
-    def test_literal_false(self):
-        template = self.from_string(
-            '<input type="input" tal:attributes="checked False" />'
-            '<input type="input" tal:attributes="checked True" />'
-            '<input type="input" tal:attributes="checked None" />'
-            '<input type="input" tal:attributes="checked default" />',
-            literal_false=True,
-            )
-
+    def test_default_marker(self):
+        template = self.from_string('<span tal:replace="id(default)" />')
         self.assertEqual(
             template(),
-            '<input type="input" checked="False" />'
-            '<input type="input" checked="True" />'
-            '<input type="input" />'
-            '<input type="input" />',
+            str(id(DEFAULT_MARKER)),
             template.source
-            )
+        )
 
     def test_boolean_attributes(self):
         template = self.from_string(
-            '<input type="input" tal:attributes="checked False" />'
-            '<input type="input" tal:attributes="checked True" />'
-            '<input type="input" tal:attributes="checked None" />'
-            '<input type="input" tal:attributes="checked \'\'" />'
-            '<input type="input" tal:attributes="checked default" />'
-            '<input type="input" checked="checked" tal:attributes="checked default" />',
-            boolean_attributes=set(['checked'])
-            )
+            "\n".join((
+                '<input type="input" tal:attributes="checked False" />',
+                '<input type="input" tal:attributes="checked True" />',
+                '<input type="input" tal:attributes="checked None" />',
+                '<input type="input" tal:attributes="checked \'\'" />',
+                '<input type="input" tal:attributes="checked default" />',
+                '<input type="input" tal:attributes="dynamic_true" />',
+                '<input type="input" tal:attributes="dynamic_false" />',
+                '<input type="input" tal:attributes="checked dynamic_marker" />',  # noqa: E501 line too long
+                '<input type="input" checked="${dynamic_marker}" />',
+                '<input type="input" checked="${True}" />',
+                '<input type="input" checked="${False}" />',
+                '<input type="input" checked="${[]}" />',
+                '<input type="input" checked="checked" tal:attributes="checked default" />',  # noqa: E501 line too long
 
+            ))
+        )
         self.assertEqual(
-            template(),
-            '<input type="input" />'
-            '<input type="input" checked="checked" />'
-            '<input type="input" />'
-            '<input type="input" />'
-            '<input type="input" />'
-            '<input type="input" checked="checked" />',
-            template.source
-            )
+            template(
+                dynamic_true={"checked": True},
+                dynamic_false={"checked": False},
+                dynamic_marker=template.default_marker.value,
+            ),
+            "\n".join((
+                '<input type="input" />',
+                '<input type="input" checked="checked" />',
+                '<input type="input" />',
+                '<input type="input" />',
+                '<input type="input" />',
+                '<input type="input" checked="checked" />',
+                '<input type="input" />',
+                '<input type="input" />',
+                '<input type="input" />',
+                '<input type="input" checked="checked" />',
+                '<input type="input" />',
+                '<input type="input" />',
+                '<input type="input" checked="checked" />',
+            )),
+            "Output mismatch\n" + template.source
+        )
 
     def test_default_debug_flag(self):
         from chameleon.config import DEBUG_MODE
         template = self.from_file(
             os.path.join(self.root, 'inputs', 'hello_world.pt'),
-            )
+        )
         self.assertEqual(template.debug, DEBUG_MODE)
         self.assertTrue('debug' not in template.__dict__)
 
@@ -468,7 +633,7 @@ class ZopePageTemplatesTest(RenderTestCase):
         template = self.from_file(
             os.path.join(self.root, 'inputs', 'hello_world.pt'),
             debug=True,
-            )
+        )
         self.assertTrue(template.debug)
         self.assertTrue(isinstance(template.loader, ModuleLoader))
 
@@ -481,12 +646,37 @@ class ZopePageTemplatesTest(RenderTestCase):
             <div metal:fill-slot="name"></dav>
             </div>
             """)
-        except ParseError:
-            exc = sys.exc_info()[1]
+        except ParseError as exc:
             self.assertTrue("</dav>" in str(exc))
         else:
             self.fail("Expected error.")
 
+    def test_f_strings(self):
+        from math import pi
+        from math import sin
+        template = self.from_string('${f"sin({a}) is {sin(a):.3}"}')
+        rendered = template(sin=sin, a=pi)
+        self.assertEqual('sin(3.141592653589793) is 1.22e-16', rendered)
+
+    def test_windows_line_endings(self):
+        template = self.from_string('<span id="span_id"\r\n'
+                                    '      class="foo"\r\n'
+                                    '      tal:content="string:bar"/>')
+        self.assertEqual(template(),
+                         '<span id="span_id"\n'
+                         '      class="foo">bar</span>')
+
+    def test_digest(self):
+        # Make sure ``digest`` doesn't error out when ``filename`` is something
+        # other than a simple string
+        data = '<html></html>'
+        template = self.from_string(data)
+        template.filename = None
+        self.assertTrue(template.digest(data, []))
+
+        template.filename = ''
+        self.assertTrue(template.digest(data, []))
+
 
 class ZopeTemplatesTestSuite(RenderTestCase):
     def setUp(self):
@@ -499,7 +689,7 @@ class ZopeTemplatesTestSuite(RenderTestCase):
     def test_pt_files(self):
         from ..zpt.template import PageTemplateFile
 
-        class Literal(object):
+        class Literal:
             def __init__(self, s):
                 self.s = s
 
@@ -519,7 +709,7 @@ class ZopeTemplatesTestSuite(RenderTestCase):
             content="<div>Hello world!</div>",
             message=Message(),
             load=loader.bind(PageTemplateFile),
-            )
+        )
 
     def test_txt_files(self):
         from ..zpt.template import PageTextTemplateFile
@@ -536,7 +726,7 @@ class ZopeTemplatesTestSuite(RenderTestCase):
 
             if mapping:
                 default = re.sub(r'\${([a-z_]+)}', r'%(\1)s', default) % \
-                          mapping
+                    mapping
 
             if target_language is None:
                 return default
@@ -546,50 +736,59 @@ class ZopeTemplatesTestSuite(RenderTestCase):
             else:
                 with_domain = " with domain '%s'" % domain
 
+            if context is None:
+                with_context = ""
+            else:
+                with_context = ", context '%s'" % context
+
             stripped = default.rstrip('\n ')
-            return "%s ('%s' translation into '%s'%s)%s" % (
-                stripped, msgid, target_language, with_domain,
+            return "{} ('{}' translation into '{}'{}{}){}".format(
+                stripped, msgid, target_language, with_domain, with_context,
                 default[len(stripped):]
-                )
+            )
 
         for input_path, output_path, language in self.find_files(ext):
             # Make friendly title so we can locate the generated
             # source when debugging
             self.shortDescription = lambda: input_path
 
-            # Very implicitly enable implicit translation based on
-            # a string included in the input path:
+            # When input path contains the string 'implicit-i18n', we
+            # enable "implicit translation".
             implicit_i18n = 'implicit-i18n' in input_path
             implicit_i18n_attrs = ("alt", "title") if implicit_i18n else ()
 
+            enable_data_attributes = 'data-attributes' in input_path
+
             template = factory(
                 input_path,
                 keep_source=True,
                 strict=False,
                 implicit_i18n_translate=implicit_i18n,
                 implicit_i18n_attributes=implicit_i18n_attrs,
-                )
+                enable_data_attributes=enable_data_attributes,
+            )
 
             params = kwargs.copy()
             params.update({
                 'translate': translate,
                 'target_language': language,
-                })
+            })
 
             template.cook_check()
 
             try:
                 got = template.render(**params)
-            except:
+            except BaseException:
                 import traceback
                 e = traceback.format_exc()
-                self.fail("%s\n\n    Example source:\n\n%s" % (e, "\n".join(
-                    ["%#03.d%s" % (lineno + 1, line and " " + line or "")
-                     for (lineno, line) in
-                     enumerate(template.source.split(
-                         '\n'))])))
+                self.fail("{}\n\n    Example source:\n\n{}".format(
+                    e,
+                    "\n".join(
+                        ["%#03.d%s" % (lineno + 1, line and " " + line or "")
+                         for (lineno, line) in enumerate(template.source.split(
+                            '\n'))])))
 
-            if isinstance(got, byte_string):
+            if isinstance(got, bytes):
                 got = got.decode('utf-8')
 
             from doctest import OutputChecker
@@ -601,23 +800,26 @@ class ZopeTemplatesTestSuite(RenderTestCase):
                 with open(output_path, 'rb') as f:
                     output = f.read()
 
-            from chameleon.utils import read_xml_encoding
             from chameleon.utils import detect_encoding
+            from chameleon.utils import read_xml_encoding
 
             if template.content_type == 'text/xml':
                 encoding = read_xml_encoding(output) or \
-                           template.default_encoding
+                    template.default_encoding
             else:
                 content_type, encoding = detect_encoding(
                     output, template.default_encoding)
 
-            want = output.decode(encoding)
+            # Newline normalization across platforms
+            want = '\n'.join(output.decode(encoding).splitlines())
+            got = '\n'.join(got.splitlines())
 
             if checker.check_output(want, got, 0) is False:
                 from doctest import Example
                 example = Example(input_path, want)
                 diff = checker.output_difference(
                     example, got, 0)
-                self.fail("(%s) - \n%s\n\nCode:\n%s" % (
+                source = template.source
+                self.fail("({}) - \n{}\n\nCode:\n{}".format(
                     input_path, diff.rstrip('\n'),
-                    template.source.encode('utf-8')))
+                    source))
